<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    ////////////////////////////////////////////////////////////////////////////////////////////
    // How do I filter on a stream, keeping only specific events?
    ////////////////////////////////////////////////////////////////////////////////////////////
    var weatherData = Application.DefineEnumerable(() => new[]
    {
        new { Timestamp = new DateTime(2010, 1, 1, 0, 00, 00, DateTimeKind.Utc), Temperature = -9.0,	StationCode = 71395,	WindSpeed = 4	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 0, 30, 00, DateTimeKind.Utc), Temperature = -4.5,	StationCode = 71801,	WindSpeed = 41	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 00, 00, DateTimeKind.Utc), Temperature = -8.8,	StationCode = 71395,	WindSpeed =	6	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 30, 00, DateTimeKind.Utc), Temperature = -4.4,	StationCode = 71801,	WindSpeed =	39	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 00, 00, DateTimeKind.Utc), Temperature = -9.7,	StationCode = 71395,	WindSpeed =	9	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 30, 00, DateTimeKind.Utc), Temperature = -4.6,	StationCode = 71801,	WindSpeed =	59	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 3, 00, 00, DateTimeKind.Utc), Temperature = -9.6,	StationCode = 71395,	WindSpeed =	9	},
    });
    
    var weather = weatherData.ToPointStreamable(
        t => PointEvent.CreateInsert(t.Timestamp, t),
        AdvanceTimeSettings.IncreasingStartTime);
    
    var filterQuery = from e in weather
                      where e.Temperature > -5
                      select e;
    
    weatherData.Count().Dump("Input event count");
        
    DisplayPoints(filterQuery, "Single condition");
    
    ////////////////////////////////////////////////////////////////////////////////////////////
    // How do I filter on multiple conditions?
    ////////////////////////////////////////////////////////////////////////////////////////////
    var query2 = from e in weather
                 where e.Temperature > -5 && e.WindSpeed > 10
                 select e;
                 
    DisplayPoints(query2, "Multiple conditions");
}

static void DisplayPoints<T>(IQStreamable<T> query, string description)
{
    (from i in query.ToPointEnumerable()
     where i.EventKind == EventKind.Insert
     select i).Dump(description);
}