<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    ////////////////////////////////////////////////////////////////////////////////	
    // How can I calculate aggregates, such as averages, over a set of events?
    ////////////////////////////////////////////////////////////////////////////////	
    
    // Some sample data to work with
    var weatherData = Application.DefineEnumerable(() => new[]
    {
        new { Timestamp = new DateTime(2010, 1, 1, 0, 00, 00, DateTimeKind.Utc), Temperature = -9.0, StationCode = 71395, WindSpeed = 4	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 0, 30, 00, DateTimeKind.Utc), Temperature = -4.5, StationCode = 71801, WindSpeed = 41 }, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 00, 00, DateTimeKind.Utc), Temperature = -8.8, StationCode = 71395, WindSpeed = 6	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 30, 00, DateTimeKind.Utc), Temperature = -4.4, StationCode = 71801, WindSpeed = 39 }, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 00, 00, DateTimeKind.Utc), Temperature = -9.7, StationCode = 71395, WindSpeed = 9 }, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 30, 00, DateTimeKind.Utc), Temperature = -4.6, StationCode = 71801, WindSpeed = 59 }, 
        new { Timestamp = new DateTime(2010, 1, 1, 3, 00, 00, DateTimeKind.Utc), Temperature = -9.6, StationCode = 71395, WindSpeed = 9	},
    });
    
    // Convert the sample data to a StreamInsight stream
    var weather = weatherData.ToPointStreamable(
        t => PointEvent.CreateInsert(t.Timestamp, t),
        AdvanceTimeSettings.IncreasingStartTime);
      
    ////////////////////////////////////////////////////////////////////////////////		
    // How can I calculate the average temperature and wind speed every 3 hours?
    ////////////////////////////////////////////////////////////////////////////////		
    var tumblingAggregates = from win in weather.TumblingWindow(TimeSpan.FromHours(3))
                             select new
                             {
                                 AverageTemperature = win.Avg(e => e.Temperature),
                                 AverageWindSpeed = win.Avg(e => e.WindSpeed),
                             };
    
    DisplayPoints(tumblingAggregates, "Average temperature & windspeed / 3 hours");
    
    ////////////////////////////////////////////////////////////////////////////////		
    // How can I calculate the average of all events in the past 3 hours, every 
    // 1 hour?
    ////////////////////////////////////////////////////////////////////////////////		
    var hoppingAggregates = from win in weather.HoppingWindow(TimeSpan.FromHours(3), TimeSpan.FromHours(1))
                            select new
                            {
                                AverageTemperature = win.Avg(e => e.Temperature),
                                AverageWindSpeed = win.Avg(e => e.WindSpeed),
                            };
    
    DisplayPoints(hoppingAggregates, "Average temperature & windspeed for the past 3 hours, update every hour");
        
    ////////////////////////////////////////////////////////////////////////////////		
    // How can I calculate the average of all events in the past 3 hours, every time 
    // a new event arrives?
    ////////////////////////////////////////////////////////////////////////////////
    var snapshotAggregates = from win in weather
                                 .AlterEventDuration(e => TimeSpan.FromHours(3))
                                 .SnapshotWindow()
                             select new
                             {
                                 AverageTemperature = win.Avg(e => e.Temperature),
                                 AverageWindSpeed = win.Avg(e => e.WindSpeed),
                                 Count = win.Count()
                             };
    
    DisplayPoints(snapshotAggregates, "Average temperature & windspeed for the past 3 hours, update on every new event");		
    
    ////////////////////////////////////////////////////////////////////////////////
    // How can I measure the rate of arriving events (per 4 hour window)
    ////////////////////////////////////////////////////////////////////////////////
    var counts = from win in weather.TumblingWindow(TimeSpan.FromHours(4))
                 select win.Count();
                 
    DisplayPoints(counts, "Rate of arriving events every 4 hours");
}

static void DisplayPoints<T>(IQStreamable<T> query, string description)
{
    (from i in query.ToPointEnumerable()
     where i.EventKind == EventKind.Insert
     select i).Dump(description);
}