<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
  <Namespace>Microsoft.ComplexEventProcessing.Extensibility</Namespace>
</Query>

void Main()
{	
    var weatherData = Application.DefineEnumerable(() => new[]
    {
        new { Timestamp = new DateTime(2010, 1, 1, 0, 00, 00, DateTimeKind.Utc), Temperature = -9.0,	StationCode = 71395,	WindSpeed = 4	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 0, 30, 00, DateTimeKind.Utc), Temperature = -4.5,	StationCode = 71801,	WindSpeed = 41	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 00, 00, DateTimeKind.Utc), Temperature = -8.8,	StationCode = 71395,	WindSpeed =	6	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 30, 00, DateTimeKind.Utc), Temperature = -4.4,	StationCode = 71801,	WindSpeed =	39	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 00, 00, DateTimeKind.Utc), Temperature = -9.7,	StationCode = 71395,	WindSpeed =	9	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 30, 00, DateTimeKind.Utc), Temperature = -4.6,	StationCode = 71801,	WindSpeed =	59	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 3, 00, 00, DateTimeKind.Utc), Temperature = -9.6,	StationCode = 71395,	WindSpeed =	9	},
    });

    var weather = weatherData.ToPointStreamable(
        t => PointEvent.CreateInsert(t.Timestamp, t),
        AdvanceTimeSettings.IncreasingStartTime);
  
    ////////////////////////////////////////////////////////////////////////////////////////////////////
    // How can I calculate the average of the past 5 events, each time a new event arrives?
    // - Note that the CountByStartTimeWindow will not produce any results until it has 
    //   been filled (in this case until 5 events have been received)
    // - Count windows only support user defined aggregates (our user defined aggregate
    //   SimpleAggregate is defined below)
    ////////////////////////////////////////////////////////////////////////////////////////////////////	
    var aggregates = from win in weather.CountWindow(5)
                     select new
                     {
                         AverageTemperature = win.UserDefinedAverage(e => e.Temperature),
                         AverageWindspeed = win.UserDefinedAverage(e => e.WindSpeed),		
                     };
    
    // Display the average values
    (from p in aggregates.ToPointEnumerable()
     where p.EventKind == EventKind.Insert
     select p).Dump("Average temperature & windspeed for the past 5 events");
}

// Define the implementation of our simple aggregate, which calculates an average
// value over the set of input elements.  
public class SimpleAggregate : CepAggregate<double, double>
{
    public override double GenerateOutput(IEnumerable<double> payloads)
    {
        return payloads.Average();
    }
}

// Define an extension method to make the user defined aggregate function 
// (SimpleAggregate.GenerateOutput) available to a query (via the extension
// method UserDefinedAverage).
public static class MyExtensions
{
    [CepUserDefinedAggregate(typeof(SimpleAggregate))]
    public static double UserDefinedAverage<InputT>(this CepWindow<InputT> window, 
        Expression<Func<InputT, double>> map)
    {
        throw CepUtility.DoNotCall();
    }
}