<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    var weatherData = Application.DefineEnumerable(() => new[]
    {
        new { Timestamp = new DateTime(2010, 1, 1, 0, 00, 00, DateTimeKind.Utc), Temperature = -9.0,	StationCode = 71395,	WindSpeed = 4	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 0, 30, 00, DateTimeKind.Utc), Temperature = -4.5,	StationCode = 71801,	WindSpeed = 41	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 00, 00, DateTimeKind.Utc), Temperature = -8.8,	StationCode = 71395,	WindSpeed =	6	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 30, 00, DateTimeKind.Utc), Temperature = -4.4,	StationCode = 71801,	WindSpeed =	39	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 00, 00, DateTimeKind.Utc), Temperature = -9.7,	StationCode = 71395,	WindSpeed =	9	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 30, 00, DateTimeKind.Utc), Temperature = -4.6,	StationCode = 71801,	WindSpeed =	59	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 3, 00, 00, DateTimeKind.Utc), Temperature = -9.6,	StationCode = 71395,	WindSpeed =	9	},
    });
    
    var weather = weatherData.ToPointStreamable(t =>
        PointEvent.CreateInsert(t.Timestamp, t),
        AdvanceTimeSettings.IncreasingStartTime);
    
    ////////////////////////////////////////////////////////////////////////////////////////////
    // How can I calculate the average of each group of events in the last 2 hours? 
    ////////////////////////////////////////////////////////////////////////////////////////////
    var query = from e in weather
                group e by e.StationCode into stationGroups
                from win in stationGroups.TumblingWindow(TimeSpan.FromHours(2))
                select new
                {	
                    StationCode = stationGroups.Key,
                    AvgTempPerStation = win.Avg(e => e.Temperature),
                    AvgWindPerStation = win.Avg(e => e.WindSpeed)
                };
    
    DisplayPoints(query, "Average temp/wind speed for each station every 2 hours");
    
    ////////////////////////////////////////////////////////////////////////////////////////////		
    // How can I calculate the average of each group of events in the last 2 hours, every hour? 
    ////////////////////////////////////////////////////////////////////////////////////////////
    var query2 = from e in weather
                 group e by e.StationCode into stationGroups
                 from win in stationGroups.HoppingWindow(TimeSpan.FromHours(2), TimeSpan.FromHours(1))
                 select new
                 {	
                     StationCode = stationGroups.Key,
                     AvgTempPerStation = win.Avg(e => e.Temperature),
                     AvgWindPerStation = win.Avg(e => e.WindSpeed)
                 };
    
    DisplayPoints(query2, "Average temp/wind speed for each station in the past 2 hours, every hour");
    
    ////////////////////////////////////////////////////////////////////////////////////////////		
    // How can I calculate the average of each group of events in the last 2 hours, every time
    // a new event in that group arrives? 
    ////////////////////////////////////////////////////////////////////////////////////////////
    var query3 = from e in weather.AlterEventDuration(e => TimeSpan.FromHours(2))
                 group e by e.StationCode into stationGroups
                 from win in stationGroups.SnapshotWindow()
                 select new
                 {	
                     StationCode = stationGroups.Key,
                     AvgTempPerStation = win.Avg(e => e.Temperature),
                     AvgWindPerStation = win.Avg(e => e.WindSpeed)
                 };
    
    DisplayIntervals(query3, "Average temp/wind speed for each station in the past 2 hours, every time a new event arrives");
        
    ////////////////////////////////////////////////////////////////////////////////////////////		
    // How can I calculate the number of groups?
    // - Create the grouping query (with appropriate time window)
    // - Each time a set of groups is produced from the first query, count the number of 
    //   events (i.e. each group will emit one event)
    ////////////////////////////////////////////////////////////////////////////////////////////
    var query4 = from e in weather
                 group e by e.StationCode into stationGroups
                 from win in stationGroups.TumblingWindow(TimeSpan.FromHours(2))
                 select new
                 {	
                     StationCode = stationGroups.Key,
                     // This aggregate is not used and is discarded in the next query
                     _ = win.Count()
                 };
    
    var counts = from win in query4.SnapshotWindow()
                 select new
                 {
                     GroupCount = win.Count()
                 };
    
    DisplayPoints(counts, "Number of groups in the group & apply query");
}

static void DisplayIntervals<T>(IQStreamable<T> query, string description)
{
    (from i in query.ToIntervalEnumerable()
     where i.EventKind == EventKind.Insert
     select i).Dump(description);
}

static void DisplayPoints<T>(IQStreamable<T> query, string description)
{
    (from i in query.ToPointEnumerable()
     where i.EventKind == EventKind.Insert
     select i).Dump(description);
}