<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    var weatherData = Application.DefineEnumerable(() => new[]
    {
        new { Timestamp = new DateTime(2010, 1, 1, 0, 00, 00, DateTimeKind.Utc), Temperature = -9.0,	StationCode = 71395,	WindSpeed = 4	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 0, 30, 00, DateTimeKind.Utc), Temperature = -4.5,	StationCode = 71801,	WindSpeed = 41	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 00, 00, DateTimeKind.Utc), Temperature = -8.8,	StationCode = 71395,	WindSpeed =	6	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 30, 00, DateTimeKind.Utc), Temperature = -4.4,	StationCode = 71801,	WindSpeed =	39	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 00, 00, DateTimeKind.Utc), Temperature = -9.7,	StationCode = 71395,	WindSpeed =	9	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 30, 00, DateTimeKind.Utc), Temperature = -4.6,	StationCode = 71801,	WindSpeed =	59	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 3, 00, 00, DateTimeKind.Utc), Temperature = -9.6,	StationCode = 71395,	WindSpeed =	9	},
    });
    
    var weather = weatherData.ToPointStreamable(t =>
        PointEvent.CreateInsert(t.Timestamp, t),
        AdvanceTimeSettings.IncreasingStartTime);
    
    ////////////////////////////////////////////////////////////////////////////////////////////
    // How can I find the events with the largest value every 4 hours?
    ////////////////////////////////////////////////////////////////////////////////////////////
    var query = from win in weather.TumblingWindow(TimeSpan.FromHours(4))
                from e in (from e in win
                            orderby e.Temperature descending
                            select e).Take(1)
                select e;
    
    DisplayPoints(query, "Highest temperature reading every 4 hours");
    
    ////////////////////////////////////////////////////////////////////////////////////////////
    // How can I find the events with the two smallest values in the last 8 hours, every hour?
    ////////////////////////////////////////////////////////////////////////////////////////////		
    var query2 = from win in weather.HoppingWindow(TimeSpan.FromHours(8), TimeSpan.FromHours(1))
                 from e in (from e in win
                            orderby e.Temperature ascending
                            select e).Take(2)
                 select e;
    
    DisplayPoints(query2, "Two lowest temperature reading in the past 8 hours, every hour");
}

static void DisplayPoints<T>(IQStreamable<T> query, string description)
{
    (from i in query.ToPointEnumerable()
     where i.EventKind == EventKind.Insert
     select i).Dump(description);
}