<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    var weatherData = Application.DefineEnumerable(() => new[]
    {
        new { Timestamp = new DateTime(2010, 1, 1, 0, 00, 00, DateTimeKind.Utc), Temperature = -9.0,	StationCode = 71395,	WindSpeed = 4	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 0, 30, 00, DateTimeKind.Utc), Temperature = -4.5,	StationCode = 71801,	WindSpeed = 41	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 00, 00, DateTimeKind.Utc), Temperature = -8.8,	StationCode = 71395,	WindSpeed =	6	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 1, 30, 00, DateTimeKind.Utc), Temperature = -4.4,	StationCode = 71801,	WindSpeed =	39	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 00, 00, DateTimeKind.Utc), Temperature = -9.7,	StationCode = 71395,	WindSpeed =	9	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 2, 30, 00, DateTimeKind.Utc), Temperature = -4.6,	StationCode = 71801,	WindSpeed =	59	}, 
        new { Timestamp = new DateTime(2010, 1, 1, 3, 00, 00, DateTimeKind.Utc), Temperature = -9.6,	StationCode = 71395,	WindSpeed =	9	},
    });
    var weather = weatherData.ToPointStreamable(
        t => PointEvent.CreateInsert(t.Timestamp, t),
        AdvanceTimeSettings.IncreasingStartTime);
    
    ////////////////////////////////////////////////////////////////////////////////////////////
    // How can I find the *group* of events with the highest *average* value in the last 8 hours?
    ////////////////////////////////////////////////////////////////////////////////////////////		
    // Step 1 - calculate the average values for each 8 hour window
    var avg = from e in weather
              group e by e.StationCode into stationGroups
              from win in stationGroups.TumblingWindow(TimeSpan.FromHours(8))
              select new
              {
                  StationCode = stationGroups.Key,
                  AvgTemperature = win.Avg(e => e.Temperature),
                  AvgWindspeed = win.Avg(e => e.WindSpeed)
              };
      
    // Step 2 - take the event with the highest average temperature in each window
    var highest = from win in avg.SnapshotWindow()
                  from e in (from e in win 
                             orderby e.AvgTemperature descending 
                             select e).Take(1)
                  select e;
    
    DisplayIntervals(highest, "Station with the highest average temperature every 8 hours");
        
    ////////////////////////////////////////////////////////////////////////////////////////////
    // How can I find the group of events with the *maximum* value in the last 8 hours, 
    // every 4 hours?
    ////////////////////////////////////////////////////////////////////////////////////////////			
    // Step 1 - calculate the average values for each 8 hour window
    var avg2 = from e in weather
               group e by e.StationCode into stationGroups
               from win in stationGroups.HoppingWindow(TimeSpan.FromHours(8), TimeSpan.FromHours(4))
               select new
               {
                   StationCode = stationGroups.Key,
                   MaxTemperature = win.Max(e => e.Temperature),
                   MaxWindspeed = win.Max(e => e.WindSpeed)
               };
      
    // Step 2 - take the event with the highest average temperature in each window
    var highest2 = from win in avg2.SnapshotWindow()
                   from e in (from e in win 
                              orderby e.MaxTemperature descending 
                              select e).Take(1)
                   select e;
    
    DisplayIntervals(highest2, "Station with the maximum temperature every 8 hours, updated every 4 hours");
}

static void DisplayIntervals<T>(IQStreamable<T> query, string description)
{
    (from i in query.ToIntervalEnumerable()
     where i.EventKind == EventKind.Insert
     select i).Dump(description);
}