<Query Kind="Statements">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

var source = Application.DefineEnumerable(() => new[]
{
    new { Name = "A", Count = 2, TieBreaker = 1 },
    new { Name = "A", Count = 3, TieBreaker = 2 },
    new { Name = "A", Count = 1, TieBreaker = 3 },
    new { Name = "A", Count = 2, TieBreaker = 4 },
    new { Name = "B", Count = 3, TieBreaker = 1 },
    new { Name = "B", Count = 3, TieBreaker = 2 },
    new { Name = "B", Count = 1, TieBreaker = 3 },
    new { Name = "C", Count = 2, TieBreaker = 1 },
    new { Name = "D", Count = 1, TieBreaker = 1 },
});

// Assign the same timestamp to every element of the input.
var startTime = new DateTime(2011, 1, 1, 0, 0, 0, DateTimeKind.Utc);
var input = source.ToPointStreamable(
    r => PointEvent.CreateInsert(startTime, r),
    AdvanceTimeSettings.IncreasingStartTime);

////////////////////////////////////////////////////////////////////////////////////////////////////
// This query returns, for each Name, the two rows with the highest Count values.
// In case of ties, uses the TieBreaker field to ensure only two results are seen
// per window.
////////////////////////////////////////////////////////////////////////////////////////////////////
var topTwoNoTies = from e in input
                   group e by e.Name into g
                   from e in
                       from win in g.SnapshotWindow()
                       from e in (from e in win
                                  orderby e.Count descending, e.TieBreaker
                                  select e).Take(2)
                       select e
                   select e;

topTwoNoTies.ToEnumerable().Dump("Top two, no ties");

////////////////////////////////////////////////////////////////////////////////////////////////////
// This query returns, for each Name, the row with the highest Count value.
// In case of ties, returns at most two results with the highest Count, using
// the TieBreaker field to determine which two should be returned.
////////////////////////////////////////////////////////////////////////////////////////////////////
var topTwoTies = from e in input
                 group e by e.Name into g
                 from e in
                     from win in (from win in g.SnapshotWindow()
                                  from e in (from e in win
                                             orderby e.Count descending
                                             select e).Take(1)
                                  select e).SnapshotWindow()
                     from e in (from e in win
                                orderby e.TieBreaker
                                select e).Take(2)
                     select e
                 select e;

topTwoTies.ToEnumerable().Dump("Top result, at most two ties");