<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    ///////////////////////////////////////////////////////////////////////////////
    // How do I shift / align events to specific times, such as midnight (the 
    // beginning of the current day)
    ///////////////////////////////////////////////////////////////////////////////
    
    var sourceIntervalData = Application.DefineEnumerable(() => new []
    {
        new { Id = "A", StartTime = new DateTime(2009, 10, 23, 23, 59, 1, DateTimeKind.Utc), EndTime = new DateTime(2009, 10, 23, 23, 59, 59, DateTimeKind.Utc) },
        new { Id = "A", StartTime = new DateTime(2009, 10, 23, 16, 12, 0, DateTimeKind.Utc), EndTime = new DateTime(2009, 10, 24, 16, 12, 1, DateTimeKind.Utc) },
        new { Id = "A", StartTime = new DateTime(2009, 10, 31, 23, 59, 59, DateTimeKind.Utc), EndTime = new DateTime(2009, 11, 1, 0, 0, 0, DateTimeKind.Utc) },
        new { Id = "A", StartTime = new DateTime(2009, 11, 1, 0, 0, 0, DateTimeKind.Utc), EndTime = new DateTime(2009, 11, 1, 12, 0, 0, DateTimeKind.Utc) },
        new { Id = "A", StartTime = new DateTime(2009, 11, 2, 0, 0, 1, DateTimeKind.Utc), EndTime = new DateTime(2009, 11, 2, 0, 0, 2, DateTimeKind.Utc) },
        new { Id = "A", StartTime = new DateTime(2009, 11, 3, 23, 59, 59, DateTimeKind.Utc), EndTime = new DateTime(2009, 11, 4, 0, 0, 1, DateTimeKind.Utc) },
    });
        
    var source = sourceIntervalData.ToIntervalStreamable(
        ev => IntervalEvent.CreateInsert(ev.StartTime.ToLocalTime(), ev.EndTime.ToLocalTime(), ev.Id),
        AdvanceTimeSettings.StrictlyIncreasingStartTime);
    
    // Align the start time of each event to the beginning of the day, keeping the end untouched
    var alignedStream = source.AlterEventLifetime(
	e => e.StartTime.Date, 
	e => e.EndTime - e.StartTime.Date);
    DisplayIntervals(alignedStream, "Aligned Stream, End of events not touched");
    
    // Align the events to the beginning of that day by shifting the event to preserve its lifetime
    var shiftedStream = source.AlterEventLifetime(
      e => e.StartTime.Date, 
      e => e.EndTime - e.StartTime);
    DisplayIntervals(shiftedStream, "Shifted stream to align to the beginning of the Day");
}

static void DisplayIntervals<T>(IQStreamable<T> query, string description)
{
    (from i in query.ToIntervalEnumerable()
     where i.EventKind == EventKind.Insert
     select i).Dump(description);
}