<Query Kind="Statements">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

/////////////////////////////////////////////////////////////////////////////////////
// How do I filter out duplicates in a window?
// -------------------------------------------------------------------------------
// - Look at 10 minute's worth of data (i.e. duplicates within a given time period)
// - Group by all of the fields that identify a duplicate.  
//   	* The traditional LINQ (and SQL) definition of "uniqueness" is to look at 
//        all fields in the record.  
//      * There may be instances in which you identify duplicates by a sub-set of 
//        fields.
// - Take (1) event from each group
/////////////////////////////////////////////////////////////////////////////////////
var dataWithDups = Application.DefineEnumerable(() => new []
{
    new { EventType = 'A', OrderNumber = 101, Action = 'S', Price = 8.65, Shares = 2000 },
    new { EventType = 'A', OrderNumber = 102, Action = 'S', Price = 8.60, Shares = 500 },
    new { EventType = 'A', OrderNumber = 103, Action = 'B', Price = 8.40, Shares = 1200 },
    new { EventType = 'A', OrderNumber = 104, Action = 'B', Price = 8.50, Shares = 1000 },
    new { EventType = 'A', OrderNumber = 105, Action = 'S', Price = 8.70, Shares = 500 },
    new { EventType = 'A', OrderNumber = 105, Action = 'S', Price = 8.70, Shares = 500 },	// Duplicate event
    new { EventType = 'A', OrderNumber = 104, Action = 'B', Price = 8.50, Shares = 1000 },	// Duplicate event
});

var dupStream = dataWithDups.ToPointStreamable(
    t => PointEvent.CreateInsert(DateTime.UtcNow.AddTicks(t.OrderNumber), t),
    AdvanceTimeSettings.StrictlyIncreasingStartTime);

// Group by all of the fields (this will put all "duplicate" events into their own
// group)
var distinctGroups = from e in dupStream.AlterEventDuration(e => TimeSpan.FromMinutes(10))
                     group e by new { e.EventType, e.OrderNumber, e.Action, e.Price, e.Shares};

// For each of these groups, take 1 event from each group
var noDuplicates = distinctGroups.SelectMany(apply => 
    from win in apply.SnapshotWindow()
    from t in (from t in win
               orderby t.EventType
               select t).Take(1)
    select t);

(from p in noDuplicates.ToIntervalEnumerable()
 where p.EventKind == EventKind.Insert
 select p).Dump("Duplicates Removed");

/////////////////////////////////////////////////////////////////////////////////////
// How do I count the number of distinct events in a window?
// -------------------------------------------------------------------------------
// - Look at 10 minute's worth of data (i.e. duplicates within a given time period)
// - Group by all of the fields that identify a duplicate.  
// - For each group,  the number of groups
/////////////////////////////////////////////////////////////////////////////////////

// Group by all of the fields (this will put all "duplicate" events into their own
// group)
var distinctQuery = from e in dupStream
                    group e by new { e.EventType, e.OrderNumber, e.Action, e.Price, e.Shares} into dGroups
                    from win in dGroups.TumblingWindow(TimeSpan.FromMinutes(10))
                    select new
                    {
                        // The number of events (i.e. how many duplicates)
                        EventCount = win.Count()
                    };

// Count to determine the number of groups (i.e. number of unique events)
var query2 = from win in distinctQuery.SnapshotWindow()
             select new { DistinctCount = win.Count() };	

(from p in query2.ToPointEnumerable()
 where p.EventKind == EventKind.Insert
 select p).Dump("Distinct Count");