<Query Kind="Statements">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

// Sample data - banking events
var now = DateTime.Now;
var inputEvents = Application.DefineEnumerable(() => new []
{  
    new { Timestamp = now+TimeSpan.FromMinutes(1), accountNo ="800001", branchName="43rd Street", customerName="John", 		amountWithdrawn = 100.0 },
    new { Timestamp = now+TimeSpan.FromMinutes(2), accountNo ="800002", branchName="23rd Street", customerName="Mark", 		amountWithdrawn = 250.0 },
    new { Timestamp = now+TimeSpan.FromMinutes(3), accountNo ="800003", branchName="43rd Street", customerName="Chandler", 	amountWithdrawn = 500.0 },
    new { Timestamp = now+TimeSpan.FromMinutes(4), accountNo ="800004", branchName="43rd Street", customerName="Dave", 		amountWithdrawn = 380.0 },
    new { Timestamp = now+TimeSpan.FromMinutes(4), accountNo ="800005", branchName="43rd Street", customerName="Mike", 		amountWithdrawn = 5000.0 },
    new { Timestamp = now+TimeSpan.FromMinutes(5), accountNo ="800006", branchName="23rd Street", customerName="Sarah", 	amountWithdrawn = 80.0 },
    new { Timestamp = now+TimeSpan.FromMinutes(6), accountNo ="800007", branchName="43rd Street", customerName="Ross", 		amountWithdrawn = 500.0 },
    new { Timestamp = now+TimeSpan.FromMinutes(8), accountNo ="800008", branchName="43rd Street", customerName="Joey", 		amountWithdrawn = 1500.0 }
});

var branchInfo = Application.DefineEnumerable(() => new []
{
    new { branchName = "23rd Street", city = "Some Town", zip = "11111" }
});

////////////////////////////////////////////////////////////////////////////////////////
// How do I do a left outer join?
// - inner join stream A & stream B
// - left anti-semi join (LASJ) stream A against stream B, setting the default/missing values
// - union the two streams
////////////////////////////////////////////////////////////////////////////////////////

// Create the branch info stream as a reference stream (events valid from now 
// to max time)
var branchStream = branchInfo.ToPointStreamable(
    t => PointEvent.CreateInsert(DateTime.Now, t),
    AdvanceTimeSettings.StrictlyIncreasingStartTime);
var refStream = branchStream.AlterEventDuration(e => TimeSpan.MaxValue);	
  
var inputStream = inputEvents.ToPointStreamable(
    t => PointEvent.CreateInsert(t.Timestamp, t),
    AdvanceTimeSettings.StrictlyIncreasingStartTime);

// In this case, we create an outer join to account up for the missing data 
// in the reference stream (the branch address information)
var innerJoinStream = from left in inputStream
                      join right in refStream on left.branchName equals right.branchName
                      select new
                      {
                          left.accountNo,
                          left.amountWithdrawn,
                          left.customerName,	
        
                          right.branchName,
                          branchCity = right.city,
                          branchZip = right.zip
                      };

// LASJ between the two branches for events without branch
// information
var lasjStream = from left in inputStream.LeftAntiJoin(refStream, left => left.branchName, right => right.branchName)
                 select new
                 {
                    left.accountNo,
                    left.amountWithdrawn,
                    left.customerName,	
    
                    left.branchName,
                    branchCity = default(string),
                    branchZip = default(string)
                 };		

// Union the streams back together
var leftOuterJoinStream = innerJoinStream.Union(lasjStream);		

(from e in leftOuterJoinStream.ToPointEnumerable()
 where e.EventKind == EventKind.Insert
 select e).Dump("left outer join results");