<Query Kind="Statements">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

////////////////////////////////////////////////////////////////////////////////////////////
// How can I convert a stream of point events into a stream of interval events, each interval 
// containing the current point event and the previous point event?"
////////////////////////////////////////////////////////////////////////////////////////////	
var sourceData = Application.DefineEnumerable(() => new []
{
    new { SourceId = "A", Value = 22, Status = 1, TimeStamp = new DateTime(2009, 10, 23, 16, 12, 0, DateTimeKind.Utc) },
    new { SourceId = "A", Value = 24, Status = 0, TimeStamp = new DateTime(2009, 10, 23, 16, 13, 0, DateTimeKind.Utc) },
    new { SourceId = "A", Value = 31, Status = 1, TimeStamp = new DateTime(2009, 10, 23, 16, 14, 0, DateTimeKind.Utc) },
    new { SourceId = "A", Value = 67, Status = 0, TimeStamp = new DateTime(2009, 10, 23, 16, 15, 0, DateTimeKind.Utc) },
    new { SourceId = "A", Value = 54, Status = 0, TimeStamp = new DateTime(2009, 10, 23, 16, 16, 0, DateTimeKind.Utc) },
    new { SourceId = "A", Value = 50, Status = 1, TimeStamp = new DateTime(2009, 10, 23, 16, 30, 0, DateTimeKind.Utc) },
    new { SourceId = "A", Value = 87, Status = 0, TimeStamp = new DateTime(2009, 10, 23, 16, 35, 0, DateTimeKind.Utc) },
});

var source = sourceData.ToPointStreamable(
    ev => PointEvent.CreateInsert(ev.TimeStamp.ToLocalTime(), ev),
    AdvanceTimeSettings.StrictlyIncreasingStartTime);

// (1) Expand each point and clip to the next one.
// (2) Then move each such interval forward by one tick, so that it overlaps just with the 
//     next point event.
// (3) Join these with the original stream and put together the payload of both points.
var result = from s in source
                    .AlterEventDuration(s => TimeSpan.MaxValue)
                    .ClipEventDuration(source, (s, e) => s.SourceId == e.SourceId)
                    .ShiftEventTime(e => TimeSpan.FromTicks(1))
             join e in source on s.SourceId equals e.SourceId
             select new
             {
                 StartValue = s.Value,
                 EndValue = e.Value,
                 Delta = e.Value - s.Value,
                 Duration = e.TimeStamp - s.TimeStamp
             };

(from e in result.ToIntervalEnumerable()
 where e.EventKind == EventKind.Insert
 select e).Dump("Current event and previous event");
 
////////////////////////////////////////////////////////////////////////////////////////////
// How can I determine the difference between consecutive values?
////////////////////////////////////////////////////////////////////////////////////////////

// Expand each point and clip to the next one then move each such interval 
// forward by one tick, so that it overlaps just with the next point event.
// Join these with the original stream and put together the payload of both points.
var result2 = from s in source
                    .AlterEventDuration(s => TimeSpan.MaxValue)
                    .ClipEventDuration(source, (s, e) => s.SourceId == e.SourceId)
                    .ShiftEventTime(e => TimeSpan.FromTicks(1))
              join e in source
              on s.SourceId equals e.SourceId
              select new
              {
                  StartValue = s.Value,
                  EndValue = e.Value,
                  StartTime = s.TimeStamp,
                  EndTime = e.TimeStamp,
                  ValueDelta = e.Value - s.Value,
                  TimeDelta = e.TimeStamp - s.TimeStamp
              };

result2.ToEnumerable().Dump("Difference between consecutive values");