<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    ////////////////////////////////////////////////////////////////////////////////////////////
    // How can I detect spikes in event values?
    ////////////////////////////////////////////////////////////////////////////////////////////	
    var now = DateTime.Now;
    var input = Application.DefineEnumerable(() => new []
    {
        new { Time = now + TimeSpan.FromSeconds(1), Value = 20},
        new { Time = now + TimeSpan.FromSeconds(2), Value = 30},
        new { Time = now + TimeSpan.FromSeconds(3), Value = 120},
        new { Time = now + TimeSpan.FromSeconds(4), Value = 200},
        new { Time = now + TimeSpan.FromSeconds(5), Value = 20},
        new { Time = now + TimeSpan.FromSeconds(6), Value = 110},
        new { Time = now + TimeSpan.FromSeconds(7), Value = 110},
        new { Time = now + TimeSpan.FromSeconds(8), Value = 210},
        new { Time = now + TimeSpan.FromSeconds(9), Value = 120},
        new { Time = now + TimeSpan.FromSeconds(10), Value = 130},
        new { Time = now + TimeSpan.FromSeconds(11), Value = 20},
        new { Time = now + TimeSpan.FromSeconds(12), Value = 30},
    });

    var inputStream = input.ToPointStreamable(
        t => PointEvent.CreateInsert(t.Time, new { Value = t.Value }),
        AdvanceTimeSettings.IncreasingStartTime);

    // 
    int threshold = 100;
    double ratio = 0.8;
    
    var resultStream = DetectSpikes(
        inputStream, 
        threshold, 					// Identify breaking the threshold (100)
        ratio, 						// Where more than 80% of the events exceed
        TimeSpan.FromSeconds(5),	// Look at the last 5 seconds
        TimeSpan.FromSeconds(1),	// Update every second
        e => e.Value);				// Measure the "Value" field
            
    (from e in resultStream.ToPointEnumerable()
     where e.EventKind == EventKind.Insert
     select e).Dump("Detect spikes in stream");
}

/// <summary>
/// Detect spikes in incoming stream
/// </summary>
/// <typeparam name="TInput">input stream event type</typeparam>
/// <param name="inputStream">input stream</param>
/// <param name="threshold">the threshold where the spike is defined</param>
/// <param name="ratio">the ratio the spike compared to the total event counts</param>
/// <param name="windowSize">window size to measure the event counts</param>
/// <param name="hopSize">hop size</param>
/// <param name="fieldSelector">select the field in the input event to detect the spike</param>
/// <returns>query that detects the spikes</returns>
private static IQStreamable<double> DetectSpikes<TInput>(
    IQStreamable<TInput> inputStream, int threshold, double ratio, 
    TimeSpan windowSize, TimeSpan hopSize,
    Expression<Func<TInput, int>> fieldSelector)
{
    var totalValues = from w in inputStream.HoppingWindow(windowSize, hopSize)
                      select w.Count();

    var bigValues = from w in inputStream.Where(e => fieldSelector.Compile()(e) > threshold).HoppingWindow(windowSize, hopSize)
                    select w.Count();

    var output = from total in totalValues
                 from big in bigValues
                 let v = (double)big / (double) total
                 where v >= ratio
                 select v;
    
    return output;
}