<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    ////////////////////////////////////////////////////////////////////////////////////////////
    // How can I account for empty windows or gaps in a data stream when
    // calculating aggregates?
    ////////////////////////////////////////////////////////////////////////////////////////////	
    var now = DateTime.Now;
    
    // Create some source data - note the time gap between 4:16 and 4:30
    var sourceData = Application.DefineEnumerable(() => new []
    {
        new { Status = 1, TimeStamp = new DateTime(2009, 10, 23, 4, 12, 0) },
        new { Status = 0, TimeStamp = new DateTime(2009, 10, 23, 4, 13, 0) },
        new { Status = 1, TimeStamp = new DateTime(2009, 10, 23, 4, 14, 0) },
        new { Status = 0, TimeStamp = new DateTime(2009, 10, 23, 4, 15, 0) },
        new { Status = 0, TimeStamp = new DateTime(2009, 10, 23, 4, 16, 0) },
        new { Status = 1, TimeStamp = new DateTime(2009, 10, 23, 4, 30, 0) },
        new { Status = 0, TimeStamp = new DateTime(2009, 10, 23, 4, 35, 0) },
    });
    
    var source = sourceData.ToPointStreamable(
        ev => PointEvent.CreateInsert(ev.TimeStamp.ToLocalTime(), ev),
        AdvanceTimeSettings.StrictlyIncreasingStartTime);
    
    // Calculate rolling aggregates
    var nonEmpty = (from window in source.TumblingWindow(TimeSpan.FromMinutes(5))
                    select window.Count()).AlterEventDuration(_ => TimeSpan.FromMinutes(5));
    
    // Account for the empty windows by joining against a "default" stream
    // The default events must contain the same information as the aggregate
    // above (in this case, a count field, set to 0)
    var defaultEvent = Application.DefineEnumerable(() => new[] {  0L });
    
    var defaultStream = defaultEvent.ToIntervalStreamable(
        ev => IntervalEvent.CreateInsert(DateTimeOffset.MinValue, DateTimeOffset.MaxValue, ev),
        AdvanceTimeSettings.StrictlyIncreasingStartTime);
    
    // LASJ the default stream against the live stream to identify the "gaps"
    DisplayIntervals(defaultStream, "Gap filler");
    
    var gaps = defaultStream.LeftAntiJoin(nonEmpty, (l, r) => true);
    
    // View the various results (base aggregates, gaps and union)
    DisplayIntervals(nonEmpty, "5 min rolling count - contains gaps");
    
    DisplayIntervals(gaps, "5 min rolling count - gaps");
    
    DisplayIntervals(nonEmpty.Union(gaps), "5 min rolling count - smooth (gaps accounted for)");
}

static void DisplayIntervals<T>(IQStreamable<T> query, string description)
{
    (from i in query.ToIntervalEnumerable()
     where i.EventKind == EventKind.Insert
     select i).Dump(description);
}
