<Query Kind="Statements">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

////////////////////////////////////////////////////////////////////////////////////////////
// How do I create a running aggregate over a set of events, such as all events that day, 
// and *continuously* update the result?"
////////////////////////////////////////////////////////////////////////////////////////////	
var sourceData = Application.DefineEnumerable(() => new []
{
    new { Value = 12, Status= 0, TimeStamp = new DateTime(2009, 10, 23, 9, 0, 0, DateTimeKind.Utc) },
    new { Value = 15, Status= 0, TimeStamp = new DateTime(2009, 10, 23, 10, 10, 0, DateTimeKind.Utc) },
    new { Value = 2, Status= 1, TimeStamp = new DateTime(2009, 10, 23, 11, 12, 0, DateTimeKind.Utc) },
    new { Value = 48, Status= 0, TimeStamp = new DateTime(2009, 10, 23, 12, 13, 0, DateTimeKind.Utc) },
    new { Value = 34, Status= 1, TimeStamp = new DateTime(2009, 10, 23, 13, 14, 0, DateTimeKind.Utc) },
    new { Value = 31, Status= 0, TimeStamp = new DateTime(2009, 10, 23, 14, 15, 0, DateTimeKind.Utc) },
    new { Value = 33, Status= 1, TimeStamp = new DateTime(2009, 10, 23, 15, 20, 0, DateTimeKind.Utc) },
    new { Value = 10, Status= 0, TimeStamp = new DateTime(2009, 10, 23, 16, 30, 0, DateTimeKind.Utc) },
    new { Value = 19, Status= 0, TimeStamp = new DateTime(2009, 10, 23, 17, 35, 0, DateTimeKind.Utc) },
});

var source = sourceData.ToPointStreamable(
    ev => PointEvent.CreateInsert(ev.TimeStamp.ToLocalTime(), ev),
    AdvanceTimeSettings.StrictlyIncreasingStartTime);

// The base unit of time measurement is the tick (i.e. we can convert any time measurement
// or offset to Ticks, then perform basic airthmetic (addition, subtraction).  Since we're
// looking for "running totals all day" we look at ticks per day.
var running = from win in source
                    // Alter each event to have an end time of "just before midnight"
                    .AlterEventDuration(e => TimeSpan.FromTicks(
                        TimeSpan.TicksPerDay - (e.StartTime.Ticks % TimeSpan.TicksPerDay)))
                    // Use a snapshot window to update on every new event
                    .SnapshotWindow()
              select new 
              { 
                  // Sum up all events received thus far
                  runningTotal = win.Sum(e => e.Value) 
              };

(from p in running.ToIntervalEnumerable()
 where p.EventKind == EventKind.Insert
 select p).Dump("Running daily total - update continuously");
  
////////////////////////////////////////////////////////////////////////////////////////////
// How do I create a running aggregate over a set of events, such as all events that day, 
// and update the result every hour?
// - The hopping window below takes a *sample* of the running total every hour
////////////////////////////////////////////////////////////////////////////////////////////	
var result = from win in running.HoppingWindow(TimeSpan.FromTicks(1), TimeSpan.FromHours(1))
             select new
             {
                // The window is only used to chop out the hourly points
                // there is only one event in each window, so the agg is
                // just a pass-through
                CountSoFar = win.Sum(e => e.runningTotal)
             };

(from p in result.ToPointEnumerable()
 where p.EventKind == EventKind.Insert
 select p).Dump("Running daily total - update every hour");