<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    var random = new Random();
    var startTime = DateTimeOffset.UtcNow;
    
    var xs = (from i in Application.DefineEnumerable(() => Enumerable.Range(0, 10))
              select PointEvent.CreateInsert(startTime.AddSeconds(i), random.NextDouble())).ToStreamable(
              	  AdvanceTimeSettings.StrictlyIncreasingStartTime);
    
    xs.Scan(() => new SmoothingOperator(0.5)).ToPointEnumerable().Where(p => p.EventKind == EventKind.Insert).Dump();
}

/// <summary>
/// Implements exponential smoothing.
/// </summary>
[DataContract]
public sealed class SmoothingOperator : CepPointStreamOperator<double, double>
{
    [DataMember]
    readonly double _smoothingFactor;

    [DataMember]
    double? _previousValue;

    public SmoothingOperator(double smoothingFactor)
    {
        _smoothingFactor = smoothingFactor;
    }

    public override IEnumerable<double> ProcessEvent(PointEvent<double> inputEvent)
    {
        // The result is a function of the previous result and the current input value.
        _previousValue = _previousValue.HasValue
            ? (1.0 - _smoothingFactor) * _previousValue.Value + _smoothingFactor * inputEvent.Payload
            : inputEvent.Payload;

        yield return _previousValue.Value;
    }

    public override bool IsEmpty
    {
        get { return false; }
    }
}