<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

// UDSO can be used to implement custom pattern matching. This example illustrates
// a simple pattern ("1 is not followed by 2 within 30 seconds").

void Main()
{
    DateTimeOffset startTime = new DateTime(2011, 5, 31);
  	var source = Application.DefineEnumerable(() => new[]
    {
        PointEvent.CreateInsert(startTime.AddSeconds(0), 1),
        PointEvent.CreateInsert(startTime.AddSeconds(20), 2), // matching input
        PointEvent.CreateInsert(startTime.AddSeconds(21), 1),
        PointEvent.CreateInsert(startTime.AddSeconds(40), 3),
        PointEvent.CreateCti<int>(startTime.AddSeconds(51)), // no matching input after 30 seconds!
    });
    
    var xs = source.ToStreamable();
    
    xs.Scan(() => new SimplePatternMatcher()).ToPointEnumerable().Dump();
}

/// <summary>
/// Indicates when an event with value '1' is followed by an event with value '2'
/// within thirty seconds.
/// </summary>
[DataContract]
public sealed class SimplePatternMatcher : CepPointStreamOperator<int, DateTime>
{
    [DataMember]
    DateTimeOffset? _nextCti;

    [DataMember]
    // Tracks timestamps for all events with value '1'.
    readonly Queue<DateTimeOffset> _active = new Queue<DateTimeOffset>();

    public override bool IsEmpty
    {
        // The operator is empty when we are not tracking any events with value '1'.
        get { return _active.Count == 0; }
    }

    public override DateTimeOffset? NextCti
    {
        get { return _nextCti; }
    }

    public override IEnumerable<DateTime> ProcessEvent(PointEvent<int> inputEvent)
    {
        // Produce output in response to the passage of time. Any active '1' event
        // not matched by a '2' event within thirty seconds matches the pattern.
        while (_active.Count > 0 && _active.Peek().AddSeconds(30) <= inputEvent.StartTime)
        {
            yield return _active.Dequeue().UtcDateTime;
        }

        // Update operator state based on new input event.
        if (inputEvent.EventKind == EventKind.Insert)
        {
            if (inputEvent.Payload == 1)
                _active.Enqueue(inputEvent.StartTime);
            else if (inputEvent.Payload == 2)
                _active.Clear();
        }

        // Schedule wake-up after thirty seconds so that we can produce output
        // if needed.
        if (_active.Count > 0)
        {
            _nextCti = _active.Peek().AddSeconds(30);
        }
    }
}