<Query Kind="Program">
  <Connection>
    <ID>8c50f33b-f431-47ce-ab08-85212ab1f0a3</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

// This sample demonstrates the following new features in StreamInsight V1.2:
// -  n-way join
// -  let-clause
// -  stream macros
void Main()
{
    Func<int, DateTimeOffset> t = (m) => new DateTimeOffset(2011, 1, 11, 8, 0, 0, TimeSpan.Zero).AddMinutes(m);
    
    var sourceData = Application.DefineEnumerable(() => new[]
    {
        new { SourceId = "A", Value = 12, TimeStamp = t(0) },
        new { SourceId = "A", Value = 14, TimeStamp = t(10) },
        new { SourceId = "C", Value = 21, TimeStamp = t(12) },
        new { SourceId = "B", Value = 24, TimeStamp = t(13) },
        new { SourceId = "B", Value = 22, TimeStamp = t(14) },
        new { SourceId = "B", Value = 30, TimeStamp = t(15) },
        new { SourceId = "C", Value = 12, TimeStamp = t(20) },
        new { SourceId = "A", Value = 29, TimeStamp = t(30) },
        new { SourceId = "B", Value = 18, TimeStamp = t(35) },
    });
    
    var referenceData = Application.DefineEnumerable(() => new[]
    {
        new { SourceId = "A", Threshold = 0.5, TimeStamp = t(0) },
        new { SourceId = "B", Threshold = 0.5, TimeStamp = t(0) },
        new { SourceId = "C", Threshold = 0.5, TimeStamp = t(0) },
        new { SourceId = "A", Threshold = 0.7, TimeStamp = t(21) },
        new { SourceId = "B", Threshold = 0.7, TimeStamp = t(21) },
    });
    
    var source = sourceData.ToPointStreamable(
        ev => PointEvent.CreateInsert(ev.TimeStamp, new { ev.SourceId, ev.Value }),
        AdvanceTimeSettings.StrictlyIncreasingStartTime);
    
    var reference = referenceData.ToPointStreamable(
        ev => PointEvent.CreateInsert(ev.TimeStamp, new { ev.SourceId, ev.Threshold }),
        AdvanceTimeSettings.StrictlyIncreasingStartTime);
    
    var fast = from e in source
               group e by e.SourceId into g
               from win in g.AlterEventDuration(e => TimeSpan.FromMinutes(3)).SnapshotWindow()
               select new { SourceId = g.Key, avg = win.Avg(e => e.Value) };
    
    var slow = from e in source
               group e by e.SourceId into g
               from win in g.AlterEventDuration(e => TimeSpan.FromMinutes(10)).SnapshotWindow()
               select new { SourceId = g.Key, avg = win.Avg(e => e.Value) };
    
    var result = from f in ToSignal(fast)
                 from s in ToSignal(slow)
                 from r in ToSignal(reference, e => e.SourceId)
                 let ratio = f.avg / s.avg
                 select new { ratio, alarm = ratio < r.Threshold };
                 
    result.ToIntervalEnumerable().Dump();
}

//Convert a stream to a signal.
public static IQStreamable<T> ToSignal<T>(
    IQStreamable<T> source)
{
    return source
        .AlterEventDuration(e => TimeSpan.MaxValue)
        .ClipEventDuration(source, (e1, e2) => true);
}

// Convert a stream into signals, partitioned by a payload key.
public static IQStreamable<T> ToSignal<T, K>(
    IQStreamable<T> source,
    Expression<Func<T, K>> keySelector)
{
    return source
        .AlterEventDuration(e => TimeSpan.MaxValue)
        .ClipEventDuration(source, (e1, e2) => (keySelector.Compile()(e1)).Equals(keySelector.Compile()(e2)));
}

// Convert a stream to a signal, with a specific timeout between events and a matching expression
// to identify signal partitions.
public static IQStreamable<T> ToSignal<T>(IQStreamable<T> inputstream, TimeSpan timeout, Expression<Func<T, T, bool>> matchExpression)
{
    return inputstream
        .AlterEventDuration(e => timeout)
        .ClipEventDuration(inputstream, (e1, e2) => matchExpression.Compile()(e1, e2));
}