<Query Kind="Program">
  <Connection>
    <ID>8c50f33b-f431-47ce-ab08-85212ab1f0a3</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    // create input data
    var left = Application.DefineEnumerable(() => new[]
    {
        CreateInsert(0, 2, 1),
        CreateInsert(1, 1.5, 2),
        CreateInsert(3, 4, 3),
    }).ToStreamable(AdvanceTimeSettings.IncreasingStartTime);
    
    var right = Application.DefineEnumerable(() => new[]
    {
        CreateInsert(0.5, 1.5, 1),
        CreateInsert(1.25, 1.5, 2),
        CreateInsert(3.5, 4.5, 3),
    }).ToStreamable(AdvanceTimeSettings.IncreasingStartTime);
    
    // left outer join
    var query = LeftOuterJoin(
        left, 
        right, 
        (l, r) => l == r, 
        l => new { Left = l, Right = (int?)null }, 
        (l, r) => new { Left = l, Right = (int?)r });
        
    (from p in query.ToPointEnumerable()
     where p.EventKind == EventKind.Insert
     select new { p.StartTime, p.Payload.Left, p.Payload.Right }).Dump();
}

public static IntervalEvent<int> CreateInsert(double start, double end, int payload)
{
    var startTime = new DateTimeOffset(new DateTime(2017, 1, 18), TimeSpan.Zero);    

    return IntervalEvent.CreateInsert(
        startTime.AddSeconds(start), startTime.AddSeconds(end), payload);
}

public static IQStreamable<TResult> LeftOuterJoin<TLeft, TRight, TResult>(
    IQStreamable<TLeft> left,
    IQStreamable<TRight> right,
    Expression<Func<TLeft, TRight, bool>> predicate,
    Expression<Func<TLeft, TResult>> outerSelector,
    Expression<Func<TLeft, TRight, TResult>> innerSelector)
{
    if (null == left) throw new ArgumentNullException("left");
    if (null == right) throw new ArgumentNullException("right");
    if (null == predicate) throw new ArgumentNullException("predicate");
    if (null == outerSelector) throw new ArgumentNullException("outerSelector");
    if (null == innerSelector) throw new ArgumentNullException("innerSelector");
    
    // call existing LASJ macro to compute results when there is no right hand-side
    var outer = from l in left.LeftAntiJoin(right, predicate)
                select outerSelector.Compile()(l);
    
    // inner join computes the remaining results
    var inner = from l in left
                from r in right
                where predicate.Compile()(l, r)
                select innerSelector.Compile()(l, r);
                
    return outer.Union(inner);
}