<Query Kind="Program">
  <Connection>
    <ID>ad2274e5-21dc-4e2c-9f3e-e55feb544557</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

void Main()
{
    var source = Application.DefineEnumerable(() => new[]
    {
        CreateInsert(0, 1, 1),
        CreateInsert(1.5, 9.5, 2),
        CreateInsert(4.2, 10.9, 3),
        CreateInsert(5, 8, 4),
    }).ToStreamable(AdvanceTimeSettings.IncreasingStartTime);
    
    var query1 = from x in source
                 group x by x % 2 into g
                 from win in g.TumblingWindow(TimeSpan.FromSeconds(1))
                 select new { Count = win.Count(), Key = g.Key };
                 
    var query2 = from x in source
                 group x by x % 2 into g
                 from win in TumblingWindow2(g, TimeSpan.FromSeconds(1))
                 select new { Count = win.Count(), Key = g.Key };
                 
    // Notice that our version of TumblingWindow does not chop results into 1 second windows
    // when the result has not changed.
    query1.ToIntervalEnumerable().Dump("Tumbling window");
    query2.ToIntervalEnumerable().Dump("Smart tumbling window");
}

public static IntervalEvent<int> CreateInsert(double start, double end, int payload)
{
    var startTime = new DateTimeOffset(new DateTime(2017, 1, 18), TimeSpan.Zero);    

    return IntervalEvent.CreateInsert(
        startTime.AddSeconds(start), startTime.AddSeconds(end), payload);
}

public static IQWindowedStreamable<T> TumblingWindow2<T>(IQStreamable<T> source, TimeSpan windowSize)
{
    if (source == null) throw new ArgumentNullException("source");

    // Implements a version of tumbling window in which no new windows are produced when
    // there is no new data. Implemented using SnapshotWindow and by "rounding" event timestamps
    // to window boundaries.
    return ExtendEventsToWindowBoundaries(source, windowSize).SnapshotWindow();
}

public static IQStreamable<T> ExtendEventsToWindowBoundaries<T>(IQStreamable<T> source, TimeSpan windowSize)
{
    if (source == null) throw new ArgumentNullException("source");

    // Define expressions that round timestamps to boundaries
    long windowSizeTicks = Math.Abs(windowSize.Ticks);
    Expression<Func<DateTime, DateTime>> roundDown = startTime =>
        startTime.AddTicks(-startTime.Ticks % windowSize.Ticks);
    Expression<Func<DateTime, DateTime>> roundUp = startTime =>
        startTime.AddTicks(startTime.Ticks % windowSize.Ticks);
       
    return source.AlterEventLifetime(
        e => roundDown.Compile()(e.StartTime),
        e => roundUp.Compile()(e.EndTime) - roundDown.Compile()(e.StartTime));
}