<Query Kind="Program">
  <Connection>
    <ID>8c50f33b-f431-47ce-ab08-85212ab1f0a3</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.StreamInsightContext</CustomTypeName>
    <DriverData>
      <ContextKind>StreamInsight</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

// This sample demonstrates the design and usage of LINQ macros.
//
// FoldPairs yields a new event for every two consecutive events,
// identified by a matching condition. The new event's payload is
// defined by a projection expression. The new event is a point event
// at the time of the pair's latter event.
void Main()
{
    var startTime = new DateTimeOffset(2011, 1, 25, 4, 0, 0, TimeSpan.Zero);

    var sourceData = Application.DefineEnumerable(() => new []
    {
        new { SourceId = "A", Value = 22, TimeStamp = startTime.AddMinutes(12) },
        new { SourceId = "B", Value = 24, TimeStamp = startTime.AddMinutes(13) },
        new { SourceId = "B", Value = 31, TimeStamp = startTime.AddMinutes(14) },
        new { SourceId = "A", Value = 67, TimeStamp = startTime.AddMinutes(15) },
        new { SourceId = "A", Value = 54, TimeStamp = startTime.AddMinutes(16) },
        new { SourceId = "B", Value = 50, TimeStamp = startTime.AddMinutes(30) },
        new { SourceId = "A", Value = 87, TimeStamp = startTime.AddMinutes(35) },
    });
    
    var source = sourceData.ToPointStreamable(
        ev => PointEvent.CreateInsert(ev.TimeStamp.ToLocalTime(), new { ev.SourceId, ev.Value }),
        AdvanceTimeSettings.StrictlyIncreasingStartTime);

    var result = FoldPairs(source,
                           (a, b) => a.SourceId == b.SourceId,
                           TimeSpan.MaxValue,
                           (a, b) => new { a.SourceId, diff = b.Value - a.Value });

    result.ToPointEnumerable().Where(e => e.EventKind == EventKind.Insert).Dump();
}


public static IQStreamable<TResult> FoldPairs<TStream, TResult>(
    IQStreamable<TStream> input,
    Expression<Func<TStream, TStream, bool>> predicate,
    TimeSpan timeout,
    Expression<Func<TStream, TStream, TResult>> resultSelector)
{
    var signal = input
                    .AlterEventDuration(e => timeout)
                    .ClipEventDuration(input, (f, s) => predicate.Compile()(f, s));
                    
    return from l in signal.ShiftEventTime(e => TimeSpan.FromTicks(1))
           from r in input
           where predicate.Compile()(l, r)
           select resultSelector.Compile()(l, r);
}