<Query Kind="Statements">
  <Connection>
    <ID>1e3830fd-9f17-427d-b3c8-0693c06d6e5c</ID>
    <Driver Assembly="StreamInsightLinqPad" PublicKeyToken="3d3a4b0768c9178e">StreamInsightLinqPad.StaticDriver</Driver>
    <CustomAssemblyPath>StreamInsightLinqPad.Samples.dll</CustomAssemblyPath>
    <CustomTypeName>StreamInsightLinqPad.Samples.HitchhikersGuideContext</CustomTypeName>
    <DriverData>
      <ContextKind>HitchhikersGuide</ContextKind>
      <RequiredVersion>21.0.0.0</RequiredVersion>
    </DriverData>
  </Connection>
</Query>

// Produces output whenever Toll Booth 1 and 2 have the same number of vehicles.

// Describe traffic per toll booth per example 04
var slidingAggregate = from t in TollData.AlterEventDuration(e => TimeSpan.FromMinutes(3))
                       group t by t.TollId into g
                       from win in g.SnapshotWindow()
                       select new
                       {
                           TollId = g.Key,
                           TotalToll = win.Sum(t => t.Toll),
                           Count = win.Count(),
                       };

// Filter out data for toll booths 1 and 2 specifically.
var toll1 = from a in slidingAggregate
            where a.TollId == "1"
            select a;

var toll2 = from a in slidingAggregate
            where a.TollId == "2"
            select a;

// Join on vehicle count.
var query = from a1 in toll1
            join a2 in toll2
            on a1.Count
            equals a2.Count
            select new
            {
                TollId1 = a1.TollId,
                TollId2 = a2.TollId,
                Count = a1.Count,
            };

// Display results
var sink = from i in query.ToIntervalEnumerable()
           where i.EventKind == EventKind.Insert
           select new
           {
               i.StartTime,
               i.EndTime,
               i.Payload.TollId1,
               i.Payload.TollId2,
               i.Payload.Count,
           };

sink.Dump("Results");